import { print } from 'graphql';
import { createClient } from 'graphql-ws';
import WebSocket from 'isomorphic-ws';
import { getOperationASTFromRequest, } from '@graphql-tools/utils';
function isClient(client) {
    return 'subscribe' in client;
}
export function buildGraphQLWSExecutor(clientOptionsOrClient) {
    let graphqlWSClient;
    let executorConnectionParams = {};
    if (isClient(clientOptionsOrClient)) {
        graphqlWSClient = clientOptionsOrClient;
    }
    else {
        graphqlWSClient = createClient({
            ...clientOptionsOrClient,
            webSocketImpl: WebSocket,
            lazy: true,
            connectionParams: () => {
                const optionsConnectionParams = (typeof clientOptionsOrClient.connectionParams === 'function'
                    ? clientOptionsOrClient.connectionParams()
                    : clientOptionsOrClient.connectionParams) || {};
                return Object.assign(optionsConnectionParams, executorConnectionParams);
            },
        });
        if (clientOptionsOrClient.onClient) {
            clientOptionsOrClient.onClient(graphqlWSClient);
        }
    }
    return function GraphQLWSExecutor(executionRequest) {
        const { document, variables, operationName, extensions, operationType = getOperationASTFromRequest(executionRequest).operation, } = executionRequest;
        // additional connection params can be supplied through the "connectionParams" field in extensions.
        // TODO: connection params only from the FIRST operation in lazy mode will be used (detect connectionParams changes and reconnect, too implicit?)
        if (extensions?.['connectionParams'] && typeof extensions?.['connectionParams'] === 'object') {
            executorConnectionParams = Object.assign(executorConnectionParams, extensions['connectionParams']);
        }
        const query = print(document);
        const iterableIterator = graphqlWSClient.iterate({
            query,
            variables,
            operationName,
            extensions,
        });
        if (operationType === 'subscription') {
            return iterableIterator;
        }
        return iterableIterator.next().then(({ value }) => value);
    };
}
